#ifndef AROS_MACHINE_H
#define AROS_MACHINE_H

/*
    Copyright  1995-2001, The AROS Development Team. All rights reserved.
    $Id: machine.h,v 1.1.1.1 2005/03/15 15:56:48 laire Exp $

    NOTE: This file must compile *without* any other header !

    Desc: machine.h include file for Amiga/m68k
    Lang: english
*/

/* Information generated by machine.c */
#define AROS_STACK_GROWS_DOWNWARDS 1 /* Stack direction */
#define AROS_BIG_ENDIAN            1 /* Big or little endian */
#define AROS_SIZEOFULONG           4 /* Size of an ULONG */
#define AROS_WORDALIGN             2 /* Alignment for WORD */
#define AROS_LONGALIGN             2 /* Alignment for LONG */
#define AROS_PTRALIGN              2 /* Alignment for PTR */
#define AROS_IPTRALIGN             2 /* Alignment for IPTR */
#define AROS_DOUBLEALIGN           2 /* Alignment for double */
#define AROS_WORSTALIGN            8 /* Worst case alignment */

#define AROS_GET_SYSBASE	extern struct ExecBase * SysBase;

/*
    How much do I have to add to sp to get the address of the first
    byte on the stack?
*/
#define SP_OFFSET 0

/*
    BCPL datatypes
*/
#define AROS_BPTR_TYPE	unsigned long
#define AROS_BSTR_TYPE  unsigned long

/* Macros to test/set failure of AllocEntry() */
#define AROS_ALLOCENTRY_FAILED(memType) \
	((struct MemList *)((IPTR)(memType) | 0x80ul<<(sizeof(APTR)-1)*8))
#define AROS_CHECK_ALLOCENTRY(memList) \
	(!((IPTR)(memList) & 0x80ul<<(sizeof(APTR)-1)*8))

/*
    One entry in a libraries' jumptable. For assembler compatibility, the
    field jmp should contain the code for an absolute jmp to a 32bit
    address. There are also a couple of macros which you should use to
    access the vector table from C.
*/
struct JumpVec
{
    unsigned short jmp;
    unsigned char vec[4];
};

/* Any jump to an unimplemented vector will cause an access to this address */
#define _aros_empty_vector		0xc0edbabe

/* Internal macros */
#define __AROS_ASMJMP			0x4EF9
#define __AROS_SET_VEC(v,a)             (*(ULONG*)(v)->vec=(ULONG)(a))
#define __AROS_GET_VEC(v)               ((APTR)(*(ULONG*)(v)->vec))

/* Use these to acces a vector table */
//#ifndef LIB_VECTSIZE
//#define LIB_VECTSIZE			(sizeof (struct JumpVec))
//#endif

#define __AROS_GETJUMPVEC(lib,n)        ((struct JumpVec *)(((UBYTE *)lib)-(n*LIB_VECTSIZE)))
#define __AROS_GETVECADDR(lib,n)        (__AROS_GET_VEC(__AROS_GETJUMPVEC(lib,n)))
#define __AROS_SETVECADDR(lib,n,addr)   (__AROS_SET_VEC(__AROS_GETJUMPVEC(lib,n),(APTR)(addr)))
#define __AROS_INITVEC(lib,n)           __AROS_GETJUMPVEC(lib,n)->jmp = __AROS_ASMJMP, \
					__AROS_SETVECADDR(lib,n,_aros_empty_vector)

/*
    Find the next valid alignment for a structure if the next x bytes must
    be skipped.
*/
#define AROS_ALIGN(x)        (((x)+AROS_WORSTALIGN-1)&-AROS_WORSTALIGN)

/* Prototypes */
extern void _aros_not_implemented (char *);

/* How much stack do we need ? Lots :-) */
#define AROS_STACKSIZE	100000

#ifdef __MORPHOS__


/* RawDoFmt hook */
#define RDFCALL(hook,data,dptr) ((void(*)(UBYTE __d0,APTR __a3))(hook))(data,dptr);

/* What to do with the library base in header, prototype and call */
#define __AROS_LH_BASE(basetype,basename)   basetype basename = (basetype)REG_A6
#define __AROS_LP_BASE(basetype,basename)   basetype __base = basename
#define __AROS_LC_BASE(basetype,basename)   REG_A6 = (LONG)__base
#define __AROS_LD_BASE(basetype,basename)   basetype __asm("r2")

/* How to transform an argument in header, prototype and call */
#define __AROS_LHA(type,name,reg)     type name = (type) REG_##reg
#define __AROS_LPA(type,name,reg)     type r##reg = name
#define __AROS_LCA(type,name,reg)     REG_##reg = (LONG)r##reg
#define __AROS_LDA(type,name,reg)     type
#define __AROS_UFHA(type,name,reg)    type name = (type) REG_##reg
#define __AROS_UFPA(type,name,reg)    type r##reg = name
#define __AROS_UFCA(type,name,reg)    REG_##reg = (LONG)r##reg
#define __AROS_UFDA(type,name,reg)    type

/* Prefix for library function in header, prototype and call */
#define __AROS_LH_PREFIX    /* eps */
#define __AROS_LP_PREFIX    /* eps */
#define __AROS_LC_PREFIX    /* eps */
#define __AROS_LD_PREFIX    /* eps */
#define __AROS_UFH_PREFIX   /* eps */
#define __AROS_UFP_PREFIX   /* eps */
#define __AROS_UFC_PREFIX   /* eps */
#define __AROS_UFD_PREFIX   /* eps */

#else

/* The registers */
#define D0 "d0"
#define D1 "d1"
#define D2 "d2"
#define D3 "d3"
#define D4 "d4"
#define D5 "d5"
#define D6 "d6"
#define D7 "d7"
#define A0 "a0"
#define A1 "a1"
#define A2 "a2"
#define A3 "a3"
#define A4 "a4"
#define A5 "a5"
#define A6 "a6"

#ifndef __SASC
#define __d0 __asm("d0")
#define __d1 __asm("d1")
#define __d2 __asm("d2")
#define __d3 __asm("d3")
#define __d4 __asm("d4")
#define __d5 __asm("d5")
#define __d6 __asm("d6")
#define __d7 __asm("d7")

#define __a0 __asm("a0")
#define __a1 __asm("a1")
#define __a2 __asm("a2")
#define __a3 __asm("a3")
#define __a4 __asm("a4")
#define __a5 __asm("a5")
#define __a6 __asm("a6")
#endif

/* RawDoFmt hook */
#define RDFCALL(hook,data,dptr) ((void(*)(UBYTE __d0,APTR __a3))(hook))(data,dptr);

/* What to do with the library base in header, prototype and call */
#define __AROS_LH_BASE(basetype,basename)   register basetype basename __asm("a6")
#define __AROS_LP_BASE(basetype,basename)   register void * __asm("a6")
#define __AROS_LC_BASE(basetype,basename)   basename
#define __AROS_LD_BASE(basetype,basename)   register basetype __asm("a6")

/* How to transform an argument in header, prototype and call */
#define __AROS_LHA(type,name,reg)     register type name __asm(reg)
#define __AROS_LPA(type,name,reg)     register type __asm(reg)
#define __AROS_LCA(type,name,reg)     name
#define __AROS_LDA(type,name,reg)     register type __asm(reg)
#define __AROS_UFHA(type,name,reg)    register type name __asm(reg)
#define __AROS_UFPA(type,name,reg)    register type __asm(reg)
#define __AROS_UFCA(type,name,reg)    name
#define __AROS_UFDA(type,name,reg)    register type __asm(reg)

/* Prefix for library function in header, prototype and call */
#define __AROS_LH_PREFIX    /* eps */
#define __AROS_LP_PREFIX    /* eps */
#define __AROS_LC_PREFIX    /* eps */
#define __AROS_LD_PREFIX    /* eps */
#define __AROS_UFH_PREFIX   /* eps */
#define __AROS_UFP_PREFIX   /* eps */
#define __AROS_UFC_PREFIX   /* eps */
#define __AROS_UFD_PREFIX   /* eps */

#endif

/* if this is defined, all AROS_LP*-macros will expand to nothing. */

#define __AROS_USE_MACROS_FOR_LIBCALL



#ifdef AROS_ALMOST_COMPATIBLE

#include <exec/types.h>
#include <exec/lists.h>

#else /* AROS_ALMOST_COMPATIBLE */

typedef unsigned long IPTR;


#define NEWLIST(_l)                              \
do                                               \
{                                                \
    struct List *l = (struct List *)(_l);        \
                                                 \
    l->lh_TailPred = (struct Node *)l;           \
    l->lh_Tail     = 0;                          \
    l->lh_Head     = (struct Node *)&l->lh_Tail; \
} while (0)

#define ADDHEAD(_l,_n)                                \
do                                                    \
{                                                     \
    struct Node *n = (struct Node *)(_n);             \
    struct List *l = (struct List *)(_l);             \
                                                      \
    n->ln_Succ          = l->lh_Head;                 \
    n->ln_Pred          = (struct Node *)&l->lh_Head; \
    l->lh_Head->ln_Pred = n;                          \
    l->lh_Head          = n;                          \
} while (0)

#define ADDTAIL(_l,_n)                                    \
do                                                        \
{                                                         \
    struct Node *n = (struct Node *)(_n);                 \
    struct List *l = (struct List *)(_l);                 \
                                                          \
    n->ln_Succ              = (struct Node *)&l->lh_Tail; \
    n->ln_Pred              = l->lh_TailPred;             \
    l->lh_TailPred->ln_Succ = n;                          \
    l->lh_TailPred          = n;                          \
} while (0)

#define REMOVE(_n)                        \
do                                        \
{                                         \
    struct Node *n = (struct Node *)(_n); \
                                          \
    n->ln_Pred->ln_Succ = n->ln_Succ;     \
    n->ln_Succ->ln_Pred = n->ln_Pred;     \
} while (0)

#ifdef __GNUC__
#   define GetHead(_l)                                       \
    ({                                                       \
        struct List *l = (struct List *)(_l);                \
                                                             \
        l->lh_Head->ln_Succ ? l->lh_Head : (struct Node *)0; \
    })

#   define GetTail(_l)                                               \
    ({                                                               \
        struct List *l = (struct List *)(_l);                        \
                                                                     \
        l->lh_TailPred->ln_Pred ? l->lh_TailPred : (struct Node *)0; \
    })

#   define GetSucc(_n)                                       \
    ({                                                       \
        struct Node *n = (struct Node *)(_n);                \
                                                             \
        n->ln_Succ->ln_Succ ? n->ln_Succ : (struct Node *)0; \
    })

#   define GetPred(_n)                                       \
    ({                                                       \
        struct Node *n = (struct Node *)(_n);                \
                                                         \
        n->ln_Pred->ln_Pred ? n->ln_Pred : (struct Node *)0; \
    })

#else

#   define GetHead(l)                                                                            \
    (                                                                                            \
        ((struct List *)(l))->lh_Head->ln_Succ ? ((struct List *)l)->lh_Head : (struct Node *)0; \
    )

#   define GetTail(l)                                                                                      \
    (                                                                                                      \
        ((struct List *)(l))->lh_TailPred->ln_Pred ? ((struct List *)(l))->lh_TailPred : (struct Node *)0; \
    )

#   define GetSucc(n)                                                                          \
    (                                                                                          \
        ((struct Node *)n)->ln_Succ->ln_Succ ? ((struct Node *)n)->ln_Succ : (struct Node *)0; \
    }

#   define GetPred(n)                                                                          \
    (                                                                                          \
        ((struct Node *)n)->ln_Pred->ln_Pred ? ((struct Node *)n)->ln_Pred : (struct Node *)0; \
    }
#endif

#define REMTAIL(_l)                       \
({                                        \
    struct List *l = (struct List *)_l;   \
    struct Node *n;                       \
    if ((n=GetTail(l)))                   \
    {                                     \
        n->ln_Pred->ln_Succ = n->ln_Succ; \
        n->ln_Succ->ln_Pred = n->ln_Pred; \
    }                                     \
    n;                                    \
})


#define ForeachNode(l,n)                       \
for                                            \
(                                              \
    n=(void *)(((struct List *)(l))->lh_Head); \
    ((struct Node *)(n))->ln_Succ;             \
    n=(void *)(((struct Node *)(n))->ln_Succ)  \
)

#define ForeachNodeSafe(l,n,n2)                 \
for                                             \
(                                               \
    n=(void *)(((struct List *)(l))->lh_Head);  \
    (n2=(void *)((struct Node *)(n))->ln_Succ); \
    n=(void *)n2                                \
)

#define SetNodeName(node,name)   \
    (((struct Node *)(node))->ln_Name = (char *)(name))
#define GetNodeName(node)        \
    (((struct Node *)(node))->ln_Name)

#define ListLength(list,count)     \
do {		                   \
    struct Node * n;	           \
    count = 0;		           \
    ForeachNode (list,n) count ++; \
} while (0)

#endif /* AROS_ALMOST_COMPATIBLE */


#endif /* AROS_MACHINE_H */
